PANIC = { } ;
function PANIC:new(o)
	o = o or {}
	setmetatable(o, self)
	self.__index = self
	return o
end

function PANIC:dump(o)
   if type(o) == 'table' then
      local s = '{ '
      for k,v in pairs(o) do
         if type(k) ~= 'number' then k = '"'..k..'"' end
         s = s .. '['..k..'] = ' .. self:dump(v) .. ','
      end
      return s .. '} '
   else
      return tostring(o)
   end
end


function PANIC:init()
	local me = self
	self.config = {}
	self.config.debug = true
	self.config.icon = "Interface\\Addons\\PANIC\\icons\\panic.tga"
	self.config.hpPanic = 0.15 --15%
	self.config.timeToShowPanic = 1.0 -- NOT USED display the panic for 1 second
	self.config.iconSize = 128
	self.config.maxNumSpells = 5
	self.config.healthstoneSpellId = 5512


	self.config.spells = {
		Druid = {
			Guardian = {
				"Survival Instincts",
				"Frienzied Regeneration",
				"Barkskin",
				"Healthstone"
			},
			Feral = {
				"Survival Instincts",
				"Renewal",
				"Healthstone",
				"Dash"
			},
			Restoration = {
				"Swiftmend",
				"Renewal",
				"Ironbark",
				"Healthstone",
				"Rejuvenation"
			}
		},
		Shaman = {
			Elemental = {
				"Astral Shift",
				"Healthstone"
			},
			Enhancement = {
				"Astral Shift",
				"Healthstone",
				"Healing Surge"
			}
		},
		Priest = {
			Holy = {
				"Desperate Prayer",
				"Healthstone"
			},
			Discipline = {
				"Power Word: Shield",
				"Healthstone"
			}
		},
		Paladin = {
			Retribution = {
				"Healthstone"
			}
		}
	}

	self.buttons = {}

	for i = 1, self.config.maxNumSpells do
		self.buttons[i] = PANICButton:new({config = self.config, i = i})
		self.buttons[i]:create()
	end

	self.isPanicking = false

	self.maxHealth = UnitHealthMax("player")

	self.totalElapsed = 0
	self.myGuid = UnitGUID("player")
	self.mainFrame = CreateFrame("Frame", "Frame", UIParent)
	self.mainFrame:SetAllPoints()
	
	local className, classFileName, classIndex = UnitClass("player")
	self.className = className
	
	local currentSpec = GetSpecialization()
	local specId, specName = GetSpecializationInfo(currentSpec)
	self.specName = specName

	self:refreshButtons()

	--print(className.. " ".. specName)

	
end

function PANIC:refreshButtons()
	--move spells
	for i = 1, self.config.maxNumSpells do
		--self.buttons[i]:moveOut()
		self.buttons[i]:clear()
	end

	if (self.config.spells[self.className] and self.config.spells[self.className][self.specName]) then

		local spells = self.config.spells[self.className][self.specName]

		local spellLookup = {}

		spells, spellLookup = self:filterSpells(spells)

		for i, spell in ipairs(spells) do
			self.buttons[i]:prepare(spell, spellLookup[spell])
			--self.buttons[i]:show()
		end

		-- timer event - to refresh every 0.5 secs
		local me = self
		self.mainFrame:SetScript("OnUpdate", function(self, elapsed)
			me:checkHP(elapsed)
		end)
	end
	--self:hide()
end

function PANIC:filterSpells(spells)
	local newSpells = {}
	local spellLookup = {}
	local j = 1;
	for i, spell in ipairs(spells) do
		local name, rank, icon, castingTime, minRange, maxRange, spellID = GetSpellInfo(spell)
		if (name or spell == 'Healthstone') then
			newSpells[j] = spell
			j = j + 1
		end
		if (icon) then
			spellLookup[spell] = icon
		else
			spellLookup[spell] = self.config.icon
		end
		--print(spell .. ': ' .. spellLookup[spell])
	end
	return newSpells, spellLookup
end

function PANIC:reset()
	self.totalElapsed = 0
end

function PANIC:hide()
	--print("hiding")
	for i, button in ipairs(self.buttons) do
		button:hide()
	end
end

function PANIC:show()
	--print("showing")
	local spellIndex = 0
	for i, button in ipairs(self.buttons) do
		local spell = button:getSpell()

		if (spell) then
			--print("spell.. ".. spell)
			--check if currently active
			--if this is a spell
			local start, duration, enable
			if (spell == "Healthstone") then
				start, duration, enable = GetItemCooldown(self.config.healthstoneSpellId)
				--print(start .. ' ' ..duration..' '..enable)
				if (start == 0 and GetItemCount(self.config.healthstoneSpellId) > 0) then
					spellIndex = i
					break
			    else
					--print(spell .. " already has buff or on CD")
					--self.buttons[i]:hide()
		    	end
			else
				start, duration, enable = GetSpellCooldown(spell)
				if (start == 0 and not UnitBuff("player", spell)) then
					spellIndex = i
					break
			    else
					--print(spell .. " already has buff or on CD")
					--self.buttons[i]:hide()
		    	end
		    end
		end
	end
	self:hide()
	if (spellIndex > 0) then
		self.buttons[spellIndex]:show()
	end
end

function PANIC:checkHP(elapsed)
	local className, classFileName, classIndex = UnitClass("player")
	
	
	local currentSpec = GetSpecialization()
	local specId, specName = GetSpecializationInfo(currentSpec)
	if (className ~= self.className or specName ~= self.specName) then
		--print("changing specs")
		self.className = className
		self.specName = specName
		self:refreshButtons()
	end


	local hp = UnitHealth("player");
	if (hp < (UnitHealthMax("player") * self.config.hpPanic)) then
			self:show()
	else
			self:hide()
	end
	--[[
	local hp = UnitHealth("player");
	if (hp > self.maxHealth) then
		self.maxHealth = hp;
	end
	if (self.currentlyPanicking) then
		self.totalElapsed = self.totalElapsed + elapsed
		if (self.totalElapsed > self.config.timeToShowPanic) then
			self.currentlyPanicking = false
			self:hide()
		else
			self:show()
		end
	else
		if (hp < (UnitHealthMax("player") * self.config.hpPanic)) then
			self.totalElapsed = 0
			self.currentlyPanicking = true
			self:show()
			--print("ZOMG PANIC!!!!!")
			--print("HP of " .. hp .. " is less than " .. self.config.hpPanic .. " * " .. self.maxHealth .. ", it's time to PANIC!!!!! at time " .. self.totalElapsed)
		end
	end
	]]
end
--intialises the addon after 5 seconds to fix a first-time-loading problem where the addon wouldnt load without this delay
--@TODO find a cleaner way to do this, especially to remove the listener because it keeps listening forever after init
function PANIC:pauseInit(delay)
	self.pauseInitTotalElapsed = 0
	if(type(delay)~="number") then
		return false;
	end
	local waitFrame = CreateFrame("Frame","WaitFrame", UIParent)
	local me = self
	print("The root of PANIC has been initialised....")
	waitFrame:SetScript("onUpdate",function (self,elapse)
		me.pauseInitTotalElapsed = me.pauseInitTotalElapsed + elapse
		if (me.pauseInitTotalElapsed > 5) then
			if (me.config == nil) then
				print("Monitoring PANIC levels...  Calm the freaking farm, until it's time to PANIC!!!!!!!!")
				me:init()
			end
		end
	end)
	return true
end

panic = PANIC:new()
panic:pauseInit(5)


